import org.springframework.context.ApplicationContext
import org.springframework.context.support.ClassPathXmlApplicationContext
import grails.spring.BeanBuilder

// simple bean
class BeanA {
	String name
}
// bean referencing other bean
class BeanB {
	String name
	BeanA parent
}
def bb = new BeanBuilder()
bb.beans {
  beanAParent(BeanA){
    name="bean A parent"
  }
}

def builder = new grails.spring.BeanBuilder(bb.createApplicationContext())

builder.beans {
  //Bean定義を書く
  
  //単純な例
  //TODO: 説明文=>大まかな共通ルール
  /*
  <bean id="exampleBean" class="ExampleBean">
    <constructor-arg value="hello exampleBean"/>
    <property name="name" value="My First Bean" />
    <property name="num" value="19" />
  </bean>
  */
  exampleBean(ExampleBean,"hello exampleBean"){
    name = "My First Bean"
    num = 19
  }
  
  //ビーンの参照
  beanA(BeanA){
    name="bean a"
  }
  //ref("...")で指定する
  beanB(BeanB){
    name="bean b"
    parent=ref("beanA")
  }
  //ref("...")は、無くても良い
  beanBB(BeanB){
    name="bean b"
    parent=beanA
  }
  //BeanBuilderのコンストラクタに指定した親ApplicationContextのBeanを参照する。
  //ref("...",true) 第2引数にtrueを指定した場合は親ApplicationContext内のみを参照します。
  beanBParent(BeanB){
    name="bean b"
    parent=ref("beanAParent",true)
  }
  
  /*
    クロージャの最初の引数は、ビーン構成用インスタンス:org.springframework.beans.factory.support.AbstractBeanDefinitionを参照しているので、
    <bean .. factoryMethod="getInstance"/>等のアトリビュートが定義できます。指定する際はAbstractBeanDefinitionのプロパティ名で指定します。
  */
  //Factoryメソッドの使用例
  withFactoryMethod(BeanWithFactoryMethod){bean->//←クロージャの最初の引数
    bean.factoryMethod = "getInstance"
    bean.singleton = false
    person = "tyama"
  }
  /*
  Bean定義の返値は、ビーン構成用インスタンスなので、戻り値の変数に指定もできます。
  */
  def somebean = withFactoryMethod2(BeanWithFactoryMethod){
    person = "yossy"
  }
  somebean.factoryMethod = "getInstance"
  somebean.singleton = false
  
  //スコープ指定
  exampleBeanScope(ExampleBean,"hello exampleBean"){bean ->
    bean.scope = "prototype" // ←プロトタイプを指定
    name = "scope prototype"
    num = 19
  }
  
  //BeanWithFactoryBean
  exampleBeanFactory(ExapleBeanFactory)
  exampleBean3(exampleBeanFactory){bean->
    bean.factoryMethod = "newInstance"
    name="example BeanFactory"
    num=55
  }
  
  //BeanWithFactoryBeanAndMethod
  exampleBean4(exampleBeanFactory:"newInstance") { bean ->
    name="example Bean with FactoryBean and Method"
    num=56
	}
	

	
  //ダイナミックにビーン名を定義
  //TODO 変数をダイナミックに変更して定義
  def beanName="dynamic"
  (1..3).each { i ->
    "${beanName}Bean${i}"(BeanA){
      name="${beanName} bean${i}"
    }
  }
  //	holder(MyExampleBeanHolder) {
  //		myExampleBeans = [
  //			ref("${beanNameBase}1"), ref("${beanNameBase}2"), ref("${beanNameBase}3")
  //		]
  //	}
  
  
  /*
  <bean id="exampleInner" class="ExampleBean2">
      <property name="bean">
        <bean class="ExampleBean">
          <property name="name" value="My First Inner Bean" />
          <property name="num" value="24" />
        </bean>
      </property>
  </bean>
  */
  //Anonymous Inner Bean
  exampleInner(ExampleBean2){
    bean = {ExampleBean b ->
      name="My First Inner Bean"
      num=24
    }
  }
  //Anonymous Inner Bean
  //Factory Beanを使った例
  exampleInnerBeanFactory(ExapleBeanFactory)
  exampleInnerUntyped(ExampleBean2){
    bean = {bean->
      bean.factoryBean = "exampleInnerBeanFactory"
      bean.factoryMethod = "newInstance"
      name="example BeanFactory"
      num=24
    }
  }
  
  /*
  <bean id="exampleAbstractBeanWiCn" abstract="true" class="ExampleBean">
    <property name="name" value="abstract bean"/>
    <property name="num" value="23"/>
  </bean>
  <bean id="exampleWiCn" parent="exampleAbstractBeanWiCn">
    <property name="name" value="My First Bean"/>
  </bean>
  */  
  //Beanの継承
  exampleAbstractBeanWiCn(ExampleBean){ bean ->
    bean.'abstract' = true
    name="abstract bean"
    num = 23
  }
  exampleWiCn("hello"){ bean ->
    bean.parent = exampleAbstractBeanWiCn
    name = "My First Bean"
  }
  
  /*
  <bean id="exampleAbstractBeanNoClass" abstract="true">
    <property name="name" value="abstract bean"/>
    <property name="num" value="20"/>
  </bean>
  <bean id="exampleNoCls" parent="exampleAbstractBeanNoClass" class="ExampleBean">
    <property name="name" value="My First Bean"/>
  </bean>
  */
  //クラス指定無しで親子関係を定義
  exampleAbstractBeanNoClass {
    name="abstract bean"
    num = 20
  }
  exampleNoCls(ExampleBean){ bean ->
    bean.parent = exampleAbstractBeanNoClass
    name = "My First Bean"
  }

}

ApplicationContext ctx = builder.createApplicationContext()
//appCtx.beanDefinitionNames.each { name ->
//	println "${name}: ${appCtx.getBean(name).toString()}"
//}

println ctx.exampleBean.num
println ctx.exampleWiCn
println ctx.exampleNoCls
println ctx.exampleInner
println ctx.exampleInnerUntyped
println ctx.withFactoryMethod
println ctx.withFactoryMethod2
println ctx.exampleBean3
println ctx.exampleBean4
println ctx.dynamicBean1.name
println ctx.dynamicBean2.name
println ctx.dynamicBean3.name

def eba = ctx.exampleBeanScope
def ebb =ctx.exampleBeanScope
if(eba!=ebb){
  println "is prototype "
}

println ctx.beanB.parent.name
println ctx.beanBB.parent.name
println ctx.beanBParent.parent.name


//ApplicationContext normalCtx = new ClassPathXmlApplicationContext("applicationContext.xml")
//println normalCtx.getBean("exampleBean")
//println normalCtx.getBean("exampleWiCn")
//println normalCtx.getBean("exampleNoCls")
//println normalCtx.getBean("exampleInner")


//BeanBuilderで記述した外部ファイルを読み込んでApplicationContextを生成する。
/*def pr = new org.springframework.core.io.support.PathMatchingResourcePatternResolver()
def r = pr.getResource("anotherConfig.groovy")

def _bb = new BeanBuilder()
_bb.loadBeans(r)

def ctx2 = _bb.createApplicationContext()
println ctx2*/